#pragma once
#include <atomic>
#include <cstdint>
#include <ctime>
#include <memory>
#include <string>
#include <unordered_map>
#include <vector>
#include <gromox/proc_common.h>
#include <gromox/simple_tree.hpp>
#include <gromox/single_list.hpp>
#include <gromox/int_hash.hpp>
#define NODE_TYPE_DOMAIN					0x81
#define NODE_TYPE_GROUP						0x82
#define NODE_TYPE_CLASS						0x83
#define NODE_TYPE_REMOTE					0x0
#define NODE_TYPE_PERSON					0x1
#define NODE_TYPE_MLIST						0x2
#define NODE_TYPE_ROOM						0x3
#define NODE_TYPE_EQUIPMENT					0x4
#define NODE_TYPE_FOLDER					0x5
#define USER_MAIL_ADDRESS					0
#define USER_REAL_NAME						1
#define USER_JOB_TITLE						2
#define USER_COMMENT						3
#define USER_MOBILE_TEL						4
#define USER_BUSINESS_TEL					5
#define USER_NICK_NAME						6
#define USER_HOME_ADDRESS					7
#define USER_CREATE_DAY						8
#define USER_STORE_PATH						9

struct PROPERTY_VALUE;

struct DOMAIN_NODE {
	SINGLE_LIST_NODE node;
	int domain_id;
	SIMPLE_TREE tree;
};

struct NSAB_NODE;
struct AB_BASE {
	AB_BASE();
	NOMOVE(AB_BASE);
	~AB_BASE() { unload(); }
	void unload();

	GUID guid{};
	std::atomic<int> status{0}, reference{0};
	time_t load_time = 0;
	int base_id = 0;
	SINGLE_LIST list, gal_list, remote_list{};
	std::unordered_map<int, NSAB_NODE *> phash;
};

struct ab_tree_del {
	void operator()(AB_BASE *);
};

using AB_BASE_REF = std::unique_ptr<AB_BASE, ab_tree_del>;

extern void ab_tree_init(const char *org_name, size_t base_size, int cache_interval, int file_blocks);
extern int ab_tree_run();
extern void ab_tree_stop();
extern AB_BASE_REF ab_tree_get_base(int base_id);
extern uint32_t ab_tree_get_leaves_num(const SIMPLE_TREE_NODE *);
extern bool ab_tree_node_to_guid(const SIMPLE_TREE_NODE *, GUID *) __attribute__((warn_unused_result));
extern BOOL ab_tree_node_to_dn(const SIMPLE_TREE_NODE *, char *buf, int len);
extern const SIMPLE_TREE_NODE *ab_tree_dn_to_node(AB_BASE *, const char *dn);
extern const SIMPLE_TREE_NODE *ab_tree_uid_to_node(const AB_BASE *, int user_id);
extern const SIMPLE_TREE_NODE *ab_tree_minid_to_node(const AB_BASE *, uint32_t minid);
extern const SIMPLE_TREE_NODE *ab_tree_minid_to_node(const AB_BASE *, uint32_t minid);
extern uint32_t ab_tree_get_node_minid(const SIMPLE_TREE_NODE *);
extern uint8_t ab_tree_get_node_type(const SIMPLE_TREE_NODE *);
extern void ab_tree_get_display_name(const SIMPLE_TREE_NODE *, uint32_t codepage, char *str_dname, size_t dn_size);
extern std::vector<std::string> ab_tree_get_object_aliases(const SIMPLE_TREE_NODE *, unsigned int type);
extern void ab_tree_get_user_info(const SIMPLE_TREE_NODE *, int type, char *value, size_t vsize);
extern void ab_tree_get_mlist_info(const SIMPLE_TREE_NODE *, char *mail_address, char *create_day, int *list_priv);
void ab_tree_get_mlist_title(uint32_t codepage, char *str_title);
extern void ab_tree_get_company_info(const SIMPLE_TREE_NODE *, char *name, char *address);
extern void ab_tree_get_department_name(const SIMPLE_TREE_NODE *, char *name);
extern void ab_tree_get_server_dn(const SIMPLE_TREE_NODE *, char *dn, int len);
int ab_tree_get_guid_base_id(GUID guid);
extern ec_error_t ab_tree_fetchprop(const SIMPLE_TREE_NODE *, unsigned int codepage, unsigned int proptag, PROPERTY_VALUE *);
extern void ab_tree_invalidate_cache();
